#!/usr/bin/env python
"""PagerDuty Saved Search Alert Script for Splunk.

Derived from @samuelks' Python Pagerduty Module
https://github.com/samuel/python-pagerduty"""
__author__ = 'Greg Albrecht <gba@splunk.com>'
__copyright__ = 'Copyright 2012 Splunk, Inc.'
__license__ = 'Apache License 2.0'


import ConfigParser
import csv
import gzip
try:
    import json
except ImportError:
    import simplejson as json
import os
import urllib2


EVENTS_URL = 'events.pagerduty.com/generic/2010-04-15/create_event.json'


class PagerDutyException(Exception):
    """Exceptions for PagerDuty objects and methods."""
    def __init__(self, status, message, errors):
        super(PagerDutyException, self).__init__(message)
        self.msg = message
        self.status = status
        self.errors = errors

    def __repr__(self):
        return "%s(%r, %r, %r)" % (
            self.__class__.__name__, self.status, self.msg, self.errors)

    def __str__(self):
        txt = "%s: %s" % (self.status, self.msg)
        if self.errors:
            txt += "\n" + "\n".join("* %s" % x for x in self.errors)
        return txt


class PagerDuty(object):
    """Class for triggering PagerDuty Incidents."""
    def __init__(self, service_key, https=True, timeout=15):
        self.service_key = service_key
        self.api_endpoint = '://'.join([('http', 'https')[https], EVENTS_URL])
        self.timeout = timeout

    def trigger(self, description, incident_key=None, details=None):
        """Triggers PagerDuty Incident"""
        return self._request(
            'trigger', description=description, incident_key=incident_key,
            details=details)

    def _request(self, event_type, **kwargs):
        """Handle PagerDuty API calls."""
        event = {'service_key': self.service_key, 'event_type': event_type}
        for kwk, kwv in kwargs.items():
            if kwv is not None:
                event[kwk] = kwv
        encoded_event = json.dumps(event)
        try:
            res = urllib2.urlopen(
                self.api_endpoint, encoded_event, self.timeout)
        except urllib2.HTTPError, exc:
            if exc.code != 400:
                raise
            res = exc

        result = json.loads(res.read())

        if result['status'] != 'success':
            raise PagerDutyException(
                result['status'], result['message'], result['errors'])

        return result.get('incident_key')


def gzcat(gzfile):
    """Decompresses gzip'd files, returns their content.

    @param gzfile: gzip'd file to decompress.
    @type gzfile: str

    @return: Contents of decompressed gzfile.
    @rtype: str
    """
    gzfd = gzip.open(gzfile, 'rb')
    file_content = gzfd.read()
    gzfd.close()
    return file_content


def csv_to_dict(raw_csv):
    """Deserializes CSV into native Python Dictionary.

    @param raw_csv: CSV content.
    @type raw_csv: str

    @return: CSV content as a native Python Dictionary.
    @rtype: dict
    """
    return csv.DictReader(raw_csv)


def extract_events():
    """Extracts event data from Splunk CSV file.

    @return: Events from CSV file.
    @rtype: list
    """
    events = []
    events_file = os.environ.get('SPLUNK_ARG_8')
    if events_file is not None:
        if os.path.exists(events_file):
            events = csv.DictReader(gzip.open(events_file))
    return events


def trigger_pagerduty(description, details, service_api_key,
                      incident_key=None):
    """Triggers PagerDuty Incident with given params.

    @param description:
    @param details:
    @param service_api_key: PagerDuty Service Integration API Key.
    @incident_key: (default=None)

    @type description: str
    @type details: str
    @type service_api_key: str
    @type incident_key: str

    @return: pagerduty.trigger object.
    @rtype: pagerduty.trigger object.
    """
    pagerduty = PagerDuty(service_api_key)
    return pagerduty.trigger(description, incident_key, details)


def get_service_api_key():
    """Extracts PagerDuty Service Integration API Key from Splunk Config.

    @return: PagerDuty Service Integration API Key.
    @rtype: str
    """
    cfg_src = os.path.join(
        os.environ['SPLUNK_HOME'], 'etc', 'apps', 'pagerduty', 'local',
        'pagerduty.conf')
    config = ConfigParser.ConfigParser()
    config.read(cfg_src)
    return config.get('service_api', 'service_api_key')


def main():
    """main, duh?"""
    # We'll serialize this dict into JSON -> PagerDuty Details.
    details = {'env': {}, 'events': []}

    service_api_key = get_service_api_key()

    for k in os.environ:
        if 'SPLUNK_ARG' in k:
            details['env'][k] = os.environ.get(k)

    events = extract_events()
    for event in events:
        details['events'].append(event)

    # This description could be any field in your event.
    description = details['events'][0]['_raw']
    trigger_pagerduty(description, details, service_api_key)


if __name__ == '__main__':
    main()
